@@ -1,45 +1,30 @@
-# Trosnoth (UberTweak Platform Game)
-# Copyright (C) 2006-2011 Joshua D Bartlett
-#
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# version 2 as published by the Free Software Foundation.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-# 02110-1301, USA.
+import logging
 
 import pygame
 
+log = logging.getLogger(__name__)
 defaultAnchor = 'topleft'
 
-def uniqueColour(colours):
-    '''Returns a greyish colour that is not in the given list of colour
-    triples. Useful for colourkeying.'''
-    result = [0, 0, 0]
-    i = 0
-    while True:
-        result[i] += 1
-        rTuple = tuple(result)
-        for colour in colours:
-            if colour == rTuple:
-                break
-        else:
-            return rTuple
-        i = (i + 1) % 3
 
 def addPositions(p1, p2):
-    return tuple([p1[i] + p2[i] for i in 0,1])
+    return tuple([p1[i] + p2[i] for i in (0, 1)])
+
 
 def canvasIntervalToScreen(app, val):
     return max(1, app.screenManager.scaleFactor * val)
 
+
+class Scalar(object):
+    def __init__(self, val):
+        self.val = val
+
+    def getVal(self, app):
+        return self.val
+
+    def __repr__(self):
+        return "Scalar(%r)" % (self.val,)
+
+
 class ScaledScalar(object):
     def __init__(self, val):
         self.val = val
@@ -56,8 +41,9 @@ class ScaledScalar(object):
     def __repr__(self):
         return "ScaledScalar (%d)" % (self.val,)
 
+
 class FullScreenAttachedPoint(object):
-    def __init__(self, distance, attachedAt = 'topleft'):
+    def __init__(self, distance, attachedAt='topleft'):
         self.val = distance
         self.attachedAt = attachedAt
 
@@ -69,8 +55,9 @@ class FullScreenAttachedPoint(object):
         pos = getattr(app.screenManager.rect, self.attachedAt)
         return addPositions(pos, val)
 
+
 class ScaledScreenAttachedPoint(object):
-    def __init__(self, distance, attachedAt = 'topleft'):
+    def __init__(self, distance, attachedAt='topleft'):
         self.val = distance
         self.attachedAt = attachedAt
 
@@ -82,12 +69,14 @@ class ScaledScreenAttachedPoint(object):
         pos = getattr(app.screenManager.scaledRect, self.attachedAt)
         return addPositions(pos, val)
 
+
 class AttachedPoint(object):
     '''
     @param attachedTo_Rect: should be a reference to a function which
             returns a pygame.Rect
     '''
-    def __init__(self, val, attachedTo_Rect, attachedAt = 'topleft'):
+
+    def __init__(self, val, attachedTo_Rect, attachedAt='topleft'):
         self.val = val
         self.attachedAt = attachedAt
         self.attachedTo_Rect = attachedTo_Rect
@@ -101,21 +90,42 @@ class AttachedPoint(object):
 
         return addPositions(pos, val)
 
+
+class RelativePoint(object):
+    '''
+    A point relative to another point.
+    '''
+
+    def __init__(self, point, size):
+        self.size = size
+        self.point = point
+
+    def __repr__(self):
+        return 'RelativePoint: (%s plus %s)' % (
+            repr(self.point), repr(self.size))
+
+    def getPoint(self, app):
+        pos = self.point.getPoint(app)
+        if hasattr(self.size, 'getSize'):
+            diff = self.size.getSize(app)
+        else:
+            diff = self.size
+        return addPositions(pos, diff)
+
+
 class FullScreenSize(object):
     def getSize(self, app):
         return app.screenManager.size
+
     def __repr__(self):
         return "Full Screen Size"
 
-class ScaledScreenSize(object):
-    def getSize(self, app):
-        return app.screenManager.scaledSize
-    def __repr__(self):
-        return "Scaled Screen Size"
 
 # An image loaded from file
 class NewImage(object):
-    def __init__(self, nameOrImage, colourkey=True):
+    def __init__(self, nameOrImage, colourkey=True, convert=True, alpha=False):
+        self._convert = convert
+        self._alpha = alpha
         if isinstance(nameOrImage, str):
             self.name = nameOrImage
             self.image = None
@@ -123,12 +133,17 @@ class NewImage(object):
             assert isinstance(nameOrImage, pygame.Surface)
             self.image = nameOrImage
         self.colourkey = colourkey
+
     def getImage(self, app):
         if self.image is not None:
             return self.image
-        self.image = pygame.image.load(self.name).convert()
+        self.image = pygame.image.load(self.name)
+        if self._alpha:
+            self.image = self.image.convert_alpha()
+        elif self._convert:
+            self.image = self.image.convert()
         if self.colourkey:
-            self.image.set_colorkey(self.image.get_at((0,0)))
+            self.image.set_colorkey(self.image.get_at((0, 0)))
         return self.image
 
     def getSize(self, app):
@@ -136,8 +151,11 @@ class NewImage(object):
 
 
 class SizedImage(NewImage):
-    def __init__(self, nameOrImage, size, colourkey=True):
-        super(SizedImage, self).__init__(nameOrImage, colourkey)
+    def __init__(
+            self, nameOrImage, size, colourkey=False, convert=False,
+            alpha=False):
+        super(SizedImage, self).__init__(
+            nameOrImage, colourkey, convert=convert, alpha=alpha)
         self.sizedImage = None
         # Keep a record of whether the size of the screen changes
         self.lastSize = None
@@ -149,7 +167,15 @@ class SizedImage(NewImage):
             return self.sizedImage
         img = super(SizedImage, self).getImage(app)
         self.lastSize = self._getCurrentSize(app)
-        self.sizedImage = pygame.transform.scale(img, self.lastSize)
+        try:
+            self.sizedImage = pygame.transform.smoothscale(
+                img, self.lastSize)
+        except Exception:
+            log.exception('Error scaling SizedImage')
+            self.sizedImage = pygame.transform.scale(
+                img, self.lastSize)
+        if self.colourkey:
+            self.sizedImage.set_colorkey(self.image.get_at((0, 0)))
         return self.sizedImage
 
     def _getCurrentSize(self, app):
@@ -162,32 +188,54 @@ class SizedImage(NewImage):
 class TextImage(NewImage):
     '''Shows text at a specified screen location.'''
 
-    def __init__(self, text, font, colour=(0,128,0), bgColour=None,
-            antialias=True):
+    def __init__(
+            self, text, font, colour=(0, 128, 0), bgColour=None,
+            antialias=True, size=None):
         self.colour = colour
         self.bgColour = bgColour
         self._text = str(text)
         self.font = font
         self.antialias = antialias
+        self.size = size
+        self.image = None
 
     def setColour(self, colour):
         self.colour = colour
+        self.image = None
 
     def setBackColour(self, colour):
         self.bgColour = colour
-    
+        self.image = None
+
     def setFont(self, font):
         self.font = font
+        self.image = None
 
     def setText(self, text):
         self._text = str(text)
+        self.image = None
+
+    def refresh(self):
+        self.image = None
+
     def getText(self):
         return self._text
     text = property(getText, setText)
 
     def getImage(self, app):
-        return self.font.render(app, self._text, self.antialias, self.colour,
-                self.bgColour)
+        if not self.image:
+            self.image = self.font.render(
+                app, self._text, self.antialias, self.colour, self.bgColour)
+
+            if self.size:
+                image = pygame.Surface(self.size, pygame.SRCALPHA)
+                image.fill((255, 255, 255, 0) if self.bgColour is None else self.bgColour)
+                r = self.image.get_rect()
+                r.center = image.get_rect().center
+                image.blit(self.image, r)
+                self.image = image
+
+        return self.image
 
 
 class Location(object):
@@ -196,6 +244,7 @@ class Location(object):
     @param  anchor  a string representing the rectangle attribute which this
                     location will reposition
     '''
+
     def __init__(self, point, anchor=defaultAnchor):
         if anchor == 'centre':
             anchor = 'center'
@@ -204,6 +253,7 @@ class Location(object):
 
     def __repr__(self):
         return "Location: %s anchored at %s" % (repr(self.point), self.anchor)
+
     def apply(self, app, rect):
         '''Moves the rect so that it is anchored according to this anchor.'''
         point = self.point
@@ -211,16 +261,17 @@ class Location(object):
             point = point.getPoint(app)
         setattr(rect, self.anchor, point)
 
+
 # Location relative to another location
 # @param size Can be a tuple or a ScaledPoint
-class RelativeLocation(Location):
+class RelativeLocation:
     def __init__(self, location, size):
         self.size = size
         self.location = location
 
     def __repr__(self):
-        return 'RelativeLocation: (%s plus %s)' % (repr(self.location),
-                repr(self.size))
+        return 'RelativeLocation: (%s plus %s)' % (
+            repr(self.location), repr(self.size))
 
     def apply(self, app, rect):
         self.location.apply(app, rect)
@@ -230,6 +281,7 @@ class RelativeLocation(Location):
             diff = self.size
         rect.topleft = addPositions(diff, rect.topleft)
 
+
 class Area(Location):
     '''
     @param  point   the point that this area is attached to
@@ -237,41 +289,65 @@ class Area(Location):
     @param  anchor  a string representing which of the rectangle's attributes
                     the point will be attached to
     '''
+
     def __init__(self, point, size, anchor=defaultAnchor):
         Location.__init__(self, point, anchor)
         self.size = size
 
     def apply(self, app, rect):
         '''Moves and resizes this rect so that it covers this area.'''
-        if hasattr(self.size, 'getSize'):
-            rect.size = self.size.getSize(app)
-        else:
-            rect.size = self.size
+        rect.size = self.getSize(app)
         Location.apply(self, app, rect)
 
     def getRect(self, app):
-        result = pygame.Rect(0,0,0,0)
+        result = pygame.Rect(0, 0, 0, 0)
         self.apply(app, result)
         return result
 
+    def getSize(self, app):
+        if hasattr(self.size, 'getSize'):
+            return self.size.getSize(app)
+        return self.size
+
     def __repr__(self):
         return "Area: point is %s, size is %s, anchored at %s" % (
-                repr(self.point), repr(self.size), self.anchor)
+            repr(self.point), repr(self.size), self.anchor)
+
+
+class LetterboxedArea:
+    def __init__(self, area, ratio=1.0):
+        self.area = area
+        self.ratio = ratio
+
+    def getRect(self, app):
+        rect = self.area.getRect(app)
+
+        # Keep anchored point fixed, but resize if needed
+        anchor = getattr(self.area, 'anchor', 'center')
+        point = getattr(rect, anchor)
+        width, height = rect.size
+        width = min(width, height * self.ratio)
+        height = min(height, width / self.ratio)
+        rect.size = (width, height)
+        setattr(rect, anchor, point)
+
+        return rect
 
 
 class PygameEvent(object):
     def __init__(self, **kwargs):
-        self._repr = 'PygameEvent(%s)' % (', '.join('%s=%s' % a for a in
-                kwargs.iteritems()))
-        for k, v in kwargs.iteritems():
+        self._repr = 'PygameEvent(%s)' % (', '.join(
+            '%s=%s' % a for a in kwargs.items()))
+        for k, v in kwargs.items():
             setattr(self, k, v)
 
     def __repr__(self):
         return self._repr
 
+
 def translateEvent(event, amount):
-    result = PygameEvent(type=event.type, pos=(event.pos[0] - amount[0],
-                                         event.pos[1] - amount[1]))
+    result = PygameEvent(type=event.type, pos=(
+        event.pos[0] - amount[0], event.pos[1] - amount[1]))
     for name in ['button', 'buttons']:
         try:
             v = getattr(event, name)
@@ -281,11 +357,13 @@ def translateEvent(event, amount):
             setattr(result, name, v)
     return result
 
+
 # For ease of use
-def ScaledLocation(x, y, anchor = defaultAnchor):
+def ScaledLocation(x, y, anchor=defaultAnchor):     # pragma: noqa
     return Location(ScaledPoint(x, y), anchor)
 
-def ScaledArea(x, y, width, height, anchor = defaultAnchor):
+
+def ScaledArea(x, y, width, height, anchor=defaultAnchor):      # pragma: noqa
     return Area(ScaledPoint(x, y), ScaledSize(width, height), anchor)
 
 
@@ -329,7 +407,7 @@ class Region(object):
 
     OTHER_ARGS = frozenset(['aspect'])     # The aspect ratio.
 
-    VALID_ARGS = OTHER_ARGS.union(STANDARD_ARGS.keys())
+    VALID_ARGS = OTHER_ARGS.union(list(STANDARD_ARGS.keys()))
 
     def __init__(self, **kwargs):
         self.aspect = None
@@ -346,17 +424,17 @@ class Region(object):
         yConstraints = {}
         constraints_2d = []
         constraints_1d = []
-        for arg, val in kwargs.iteritems():
+        for arg, val in kwargs.items():
             xKind, yKind = self.STANDARD_ARGS[arg]
             if xKind != '-':
                 if xKind in xConstraints:
-                    raise TypeError('specifying %s overconstrains region (x)' %
-                            (arg,))
+                    raise TypeError(
+                        'specifying %s overconstrains region (x)' % (arg,))
                 xConstraints[xKind] = arg
             if yKind != '-':
                 if yKind in yConstraints:
-                    raise TypeError('specifying %s overconstrains region (y)' %
-                            (arg,))
+                    raise TypeError(
+                        'specifying %s overconstrains region (y)' % (arg,))
                 yConstraints[yKind] = arg
 
             if xKind == '-':
@@ -378,7 +456,8 @@ class Region(object):
                 elif xKind != 's' and yKind != 's':
                     method = 'getPoint'
                 else:
-                    raise AssertionError('definition of %r in '
+                    raise AssertionError(
+                        'definition of %r in '
                         'Region.STANDARD_ARGS does not make sense' % (arg,))
 
             c.append((arg, val, method))
@@ -397,32 +476,33 @@ class Region(object):
                 raise TypeError('Region is underconstrained horizontally')
             if yNum < 2:
                 raise TypeError('Region is underconstrained vertically')
-            self.constraints = [self._makeXConstraint(xConstraints),
-                    self._makeYConstraint(yConstraints)]
+            self.constraints = [
+                self._makeXConstraint(xConstraints),
+                self._makeYConstraint(yConstraints)]
         else:
             if xNum + yNum > 3:
                 raise TypeError('Region is overconstrained')
             if xNum + yNum < 3:
                 raise TypeError('Region is underconstrained')
             if xNum == 2:
-                self.constraints = self._makeYAspectConstraints(xConstraints,
-                        yConstraints, aspect)
+                self.constraints = self._makeYAspectConstraints(
+                    xConstraints, yConstraints, aspect)
             else:
-                self.constraints = self._makeXAspectConstraints(xConstraints,
-                        yConstraints, aspect)
+                self.constraints = self._makeXAspectConstraints(
+                    xConstraints, yConstraints, aspect)
 
     def __repr__(self):
         return self._repr
 
     def _setRepr(self, kwargs):
-        self._repr = 'Region(%s)' % (', '.join('%s=%r' % t for t in
-                kwargs.iteritems()))
+        self._repr = 'Region(%s)' % (', '.join(
+            '%s=%r' % t for t in kwargs.items()))
 
     def _validateArgNames(self, kwargs):
         for arg in kwargs:
             if arg not in self.VALID_ARGS:
-                raise TypeError('Region() got an unexpected keyword argument '
-                        + repr(arg))
+                raise TypeError(
+                    'Region() got an unexpected keyword argument ' + repr(arg))
 
     def getRect(self, app):
         vals = self._evaluateConstraints(app)
@@ -430,6 +510,13 @@ class Region(object):
         self._applyConstraints(r, vals)
         return r
 
+    def apply(self, app, rect):
+        # Exists so a Region can be used as the arg to a
+        # RelativeLocation.
+        r = self.getRect(app)
+        rect.size = r.size
+        rect.topleft = r.topleft
+
     def _applyConstraints(self, r, vals):
         for c, args in self.constraints:
             c(r, vals, args)
@@ -497,7 +584,7 @@ class Region(object):
         setattr(r, attr, vals[key][1])
 
     def _makeXAspectConstraints(self, xConstraints, yConstraints, aspect):
-        key, val = xConstraints.items().pop()
+        key, val = list(xConstraints.items()).pop()
         if key == '0':
             attr = 'left'
         elif key == '1':
@@ -512,7 +599,7 @@ class Region(object):
         ]
 
     def _makeYAspectConstraints(self, xConstraints, yConstraints, aspect):
-        key, val = yConstraints.items().pop()
+        key, val = list(yConstraints.items()).pop()
         if key == '0':
             attr = 'top'
         elif key == '1':
@@ -523,26 +610,48 @@ class Region(object):
             raise TypeError('Region is overconstrained')
         return [
             self._makeXConstraint(xConstraints),
-            (self._yAspectConstraint, ((aspect+0.), attr, val))
+            (self._yAspectConstraint, ((aspect + 0.), attr, val))
         ]
 
     def _getIntervalFromEdges(self, x1, x2):
         return (x1, x2 - x1)
+
     def _getIntervalFromInterval(self, x1, deltaX):
         return (x1, deltaX)
+
     def _getIntervalFromFirstHalf(self, x1, midx):
         return (x1, 2 * (midx - x1))
+
     def _getIntervalFromSecondHalf(self, midx, x2):
         deltaX = 2 * (x2 - midx)
         return (x2 - deltaX, deltaX)
+
     def _getIntervalFromCentre(self, midx, deltaX):
         return midx - int(deltaX / 2), deltaX
+
     def _getIntervalFromReverseInterval(self, x2, deltaX):
         return (x2 - deltaX, deltaX)
 
+
+class PaddedRegion(object):
+    def __init__(self, region, padding):
+        self.padding = padding
+        self.region = region
+
+    def getRect(self, app):
+        rect = pygame.Rect(self.region.getRect(app))
+        padding = self.padding.getVal(app)
+        rect.width += 2 * padding
+        rect.height += 2 * padding
+        rect.top -= padding
+        rect.left -= padding
+        return rect
+
+
 class Abs(object):
     def __init__(self, *args):
         self.values = args
+
     def __repr__(self):
         return 'Abs(%s)' % ', '.join(repr(v) for v in self.values)
 
@@ -561,9 +670,11 @@ class Abs(object):
         assert len(self.values) == 4
         return pygame.Rect(*self.values)
 
+
 class Canvas(object):
     def __init__(self, *args):
         self.values = args
+
     def __repr__(self):
         return 'Canvas(%s)' % ', '.join(repr(v) for v in self.values)
 
@@ -596,9 +707,47 @@ class Canvas(object):
         xr, yr, wr, hr = self.values
         return pygame.Rect(x * xr, y * yr, x * wr, y * hr)
 
+
+class CanvasX(object):
+    '''Uses a scaled value for the X axis only'''
+
+    def __init__(self, *args):
+        self.values = args
+
+    def __repr__(self):
+        return 'CanvasX(%s)' % ', '.join(repr(v) for v in self.values)
+
+    def getVal(self, app):
+        assert len(self.values) == 1
+        return canvasIntervalToScreen(app, self.values[0])
+    getXVal = getVal
+
+    def getYVal(self, app):
+        assert len(self.values) == 1
+        return self.values[0]
+
+    def getPoint(self, app):
+        assert len(self.values) == 2
+        sm = app.screenManager
+        return (int(self.values[0] * sm.scaleFactor + sm.offsets[0]),
+                self.values[1])
+
+    def getSize(self, app):
+        assert len(self.values) == 2
+        sm = app.screenManager
+        return (int(self.values[0] * sm.scaleFactor), self.values[1])
+
+    def getRect(self, app):
+        assert len(self.values) == 4
+        x, y = app.screenManager.size
+        xr, yr, wr, hr = self.values
+        return pygame.Rect(x * xr, yr, x * wr, hr)
+
+
 class Screen(object):
     def __init__(self, *args):
         self.values = args
+
     def __repr__(self):
         return 'Screen(%s)' % ', '.join(repr(v) for v in self.values)
 
@@ -625,6 +774,7 @@ class Screen(object):
         xr, yr, wr, hr = self.values
         return pygame.Rect(x * xr, y * yr, x * wr, y * hr)
 
+
 class Relative(object):
     '''
     Provides a point relative to an existing rect.
@@ -632,6 +782,7 @@ class Relative(object):
     @param x: relative x value from 0 to 1.
     @param y: relative y value from 0 to 1.
     '''
+
     def __init__(self, rectGiver, x, y):
         self.rectGiver = rectGiver
         self.x = x
@@ -648,15 +799,16 @@ class Relative(object):
         r = self.rectGiver()
         return (self.x * r.width, self.y * r.height)
 
+
 class ScaledSize(Canvas):
     '''
     For compatibility with dodgy code like gui.framework.slider.Slider() that
     directly uses .val() of something that it's not sure is a
     ScaledSize/ScaledPoint.
     '''
+
     def __init__(self, *args):
         Canvas.__init__(self, *args)
         self.val = args
-ScaledPoint = ScaledSize
 
-Size = Abs
+ScaledPoint = ScaledSize
