@@ -1,5 +1,5 @@
 # Trosnoth (UberTweak Platform Game)
-# Copyright (C) 2006-2011 Joshua D Bartlett
+# Copyright (C) 2006-2012 Joshua D Bartlett
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -19,7 +19,7 @@ import logging
 
 import pygame
 
-log = logging.getLogger('fontcache')
+log = logging.getLogger(__name__)
 
 _cache = {}
 
