@@ -1,5 +1,5 @@
 # Trosnoth (UberTweak Platform Game)
-# Copyright (C) 2006-2011 Joshua D Bartlett
+# Copyright (C) 2006-2012 Joshua D Bartlett
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,15 +15,14 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301, USA.
 
-from framework import Element, CompoundElement
-from elements import TextButton, TextElement
-from trosnoth.gui.common import (translateEvent, Location, addPositions,
-    AttachedPoint, Relative)
+import pygame
+
+from .framework import Element, CompoundElement
+from trosnoth.gui.common import translateEvent, addPositions, AttachedPoint, Relative
 from trosnoth.gui.errors import MultiWindowException
-from trosnoth.gui.framework.utils import wrapWords
 from trosnoth.utils.event import Event
 from trosnoth.utils.utils import new
-import pygame
+
 
 class MoveableBox(CompoundElement):
     defaultBorderColour = (0,0,255)
@@ -105,8 +104,9 @@ class MoveableBox(CompoundElement):
         try:
             showDialog = self.app.screenManager.showDialog
         except AttributeError:
-            raise MultiWindowException('Dialog Boxes cannot be used unless the '
-                    'Application is a MultiWindowApplication')
+            raise MultiWindowException(
+                'Dialog Boxes cannot be used unless the Application is a '
+                'MultiWindowApplication')
         showDialog(self)
         self.showing = True
 
@@ -120,6 +120,7 @@ class MoveableBox(CompoundElement):
         self.showing = False
         self.onClose.execute()
 
+
 class DialogBox(MoveableBox):
     flashTime = 0.15
     flashNum = 2
@@ -181,70 +182,9 @@ class DialogBox(MoveableBox):
     def setCaption(self, caption=None, subCaption=None):
         MoveableBox.setCaption(self, caption, subCaption)
 
-class MessageBox(DialogBox):
-    '''buttons = a tuple of DialogResults'''
-    def __init__(self, app, size, caption, message, buttons):
-        DialogBox.__init__(self, app, size, caption)
-        if hasattr(size, 'getSize'):
-            size = size.getSize(app)
-        textFont = app.screenManager.fonts.labelFont
-        lines = wrapWords(app, message, textFont, size[0])
-        y = 5
-        for text in lines:
-            textPos = Location(DialogBoxAttachedPoint(self, (0, y), 'midtop'),
-                    'midtop')
-            self.elements.append(TextElement(app, text, textFont, textPos))
-            y += textFont.getHeight(app)
-        buttonFont = app.screenManager.fonts.captionFont
-        for x in range(0, len(buttons)):
-            dialogResult = buttons[x]
-            location = Location(DialogBoxAttachedPoint(self, (0,
-                    -textFont.getHeight(self.app) / 2), 'midbottom'),
-                    'midbottom')
-        buttonFont = app.screenManager.fonts.captionFont
-        for x in range(0, len(buttons)):
-            dialogResult = buttons[x]
-            location = Location(((x + 1) * size[0] / (len(buttons) + 1), size[1]
-                    * 2 / 3), 'midtop')
-            button = TextButton(app, location, DialogResult.text[dialogResult],
-                    buttonFont, (255,0,0), (255,100,200))
-            button._mbResult = dialogResult
-            button.onClick.addListener(self._setResultAndClose)
-            self.elements.append(button)
-        self.result = None
-
-    def _setResultAndClose(self, button):
-        self.result = button._mbResult
-        self.close()
-
-class YesNoBox(MessageBox):
-    def __init__(self, app, size, caption, message):
-        buttons = (DialogResult.Yes, DialogResult.No)
-        MessageBox.__init__(self, app, size, caption, message, buttons)
-
-class YesNoCancelBox(MessageBox):
-    def __init__(self, app, size, caption, message):
-        buttons = (DialogResult.Yes, DialogResult.No, DialogResult.Cancel)
-        MessageBox.__init__(self, app, size, caption, message, buttons)
-
-class OkBox(MessageBox):
-    def __init__(self, app, size, caption, message):
-        buttons = tuple([DialogResult.OK])
-        MessageBox.__init__(self, app, size, caption, message, buttons)
-
-class OkCancelBox(MessageBox):
-    def __init__(self, app, size, caption, message):
-        buttons = (DialogResult.OK, DialogResult.Cancel)
-        MessageBox.__init__(self, app, size, caption, message, buttons)
-
-class OkRetryBox(MessageBox):
-    def __init__(self, app, size, caption, message):
-        buttons = (DialogResult.OK, DialogResult.Retry)
-        MessageBox.__init__(self, app, size, caption, message, buttons)
 
 class DialogResult(object):
-    OK, Cancel, Yes, No, Retry = new(5)
-    text = {OK:'OK', Cancel:'Cancel', Yes:'Yes', No:'No', Retry:'Retry'}
+    OK, Cancel = new(2)
 
 
 def getInsideRectFromZero(box):
@@ -252,9 +192,11 @@ def getInsideRectFromZero(box):
     r.topleft = (0,0)
     return r
 
+
 def DialogBoxAttachedPoint(box, val, anchor='topleft'):
     return AttachedPoint(val, lambda:getInsideRectFromZero(box), anchor)
 
+
 class MoveableBoxEdge(Element):
     '''
     Handles the border (including dragging and dropping) of boxes. It also
@@ -270,9 +212,11 @@ class MoveableBoxEdge(Element):
         self.subCaption = subCaption
         self.beingDragged = False
         self.borderWidth = 3
-        appSize = app.screenManager.size
-        self._setPos(tuple([(appSize[i] - self._getFullSize()[i]) / 2 for i
-                in 0,1]))
+        self.auto_position()
+
+    def auto_position(self):
+        appSize = self.app.screenManager.size
+        self._setPos(tuple([(appSize[i] - self._getFullSize()[i]) / 2 for i in (0,1)]))
 
     def _getFullSize(self):
         return (self._getFullWidth(), self.box._getSize()[1] +
@@ -330,9 +274,9 @@ class MoveableBoxEdge(Element):
         # Make pretty edges:
         setOuterBorder = lambda x, y: min(x + y, 255)
         outerBorder1 = tuple([setOuterBorder(self.borderColour[i], 70)
-                for i in 0,1,2])
+                for i in (0,1,2)])
         outerBorder2 = tuple([setOuterBorder(self.borderColour[i], 35)
-                for i in 0,1,2])
+                for i in (0,1,2)])
         pygame.draw.rect(screen, outerBorder1, self._getEdgeRect(), 1)
         if self.borderWidth > 2:
             oldRect = self._getEdgeRect()
@@ -354,4 +298,3 @@ class MoveableBoxEdge(Element):
             height = int(self.font.getLineSize(self.app) * 0.8)
             screen.blit(subText, addPositions(self.pos, (self.borderWidth +
                     5, height)))
-
