@@ -1,20 +1,3 @@
-# Trosnoth (UberTweak Platform Game)
-# Copyright (C) 2006-2011 Joshua D Bartlett
-#
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# version 2 as published by the Free Software Foundation.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-# 02110-1301, USA.
-
 import logging
 
 import pygame
@@ -28,7 +11,8 @@ from trosnoth.gui.common import Location, AttachedPoint, addPositions, Area
 from trosnoth.utils.event import Event
 from trosnoth.utils.utils import BasicContextManager
 
-log = logging.getLogger('game')
+log = logging.getLogger(__name__)
+
 
 # Styles work thusly:
 # When looking for a particular attribute (say backColour),
@@ -37,7 +21,6 @@ log = logging.getLogger('game')
 # then in the table's. As soon as it finds a non-None value,
 # it will use that.
 class TableStyle(BasicContextManager):
-
     def __init__(self):
         self.backColour = None
         self.foreColour = None
@@ -49,14 +32,12 @@ class TableStyle(BasicContextManager):
         self.textAlign = None
 
 
-
 class StyleUser(CompoundElement):
     def __init__(self, app, style):
         super(StyleUser, self).__init__(app)
         self.style = style
 
 
-
 class CellAggregator(StyleUser):
     def __init__(self, app, table):
         super(CellAggregator, self).__init__(app, TableStyle())
@@ -93,14 +74,15 @@ class CellAggregator(StyleUser):
     def styleUpdate(self):
         '''
         Informs the row or column that the style of some part of the table has
-        changed.  This is provided because it is too expensive for every cell to
-        check its style every draw() call, so you must manually tell the table,
-        row, column or cell that the style has been updated before the update is
-        reflected.
+        changed.  This is provided because it is too expensive for every cell
+        to check its style every draw() call, so you must manually tell the
+        table, row, column or cell that the style has been updated before the
+        update is reflected.
         '''
         for cell in self._cells:
             cell.styleUpdate()
 
+
 class Row(CellAggregator):
     def __init__(self, app, table, index, height):
         super(Row, self).__init__(app, table)
@@ -118,18 +100,12 @@ class Row(CellAggregator):
     def _getPt(self):
         return self._table._getRowPt(self.index)
 
+
 class Column(CellAggregator):
     def __init__(self, app, table, width=0):
         super(Column, self).__init__(app, table)
         self.setWidth(width)
 
-    def onClicked(self, sender):
-        self.setFocus(sender)
-        self._table.onClicked(self)
-
-    def lostFocus(self):
-        self.setFocus(None)
-
     def setWidth(self, width):
         self._width = width
 
@@ -146,24 +122,34 @@ class Column(CellAggregator):
     def _getPt(self):
         return self._table._getColPt(self.index)
 
+
 class TextColumn(Column):
     def createCell(self, row):
         return TextCell(self.app, row, self)
 
+
 class TextBoxColumn(Column):
     def createCell(self, row):
         return TextBoxCell(self.app, row, self)
 
+
 class TextButtonColumn(Column):
     def createCell(self, row):
         return TextButtonCell(self.app, row, self)
 
+
 class Cell(StyleUser):
     def __init__(self, app, row, column):
         super(Cell, self).__init__(app, TableStyle())
         self._row = row
         self._column = column
-        self._changed = True
+        self._styleChanged = True
+        self.rect = None
+        self.usableRect = None
+
+    def layoutChanged(self):
+        self.rect = None
+        self.usableRect = None
 
     @property
     def rowNum(self):
@@ -176,12 +162,12 @@ class Cell(StyleUser):
     def styleUpdate(self):
         '''
         Informs the cell that the style of some part of the table has changed.
-        This is provided because it is too expensive for every cell to check its
-        style every draw() call, so you must manually tell the table, row,
+        This is provided because it is too expensive for every cell to check
+        its style every draw() call, so you must manually tell the table, row,
         column or cell that the style has been updated before the update is
         reflected.
         '''
-        self._changed = True
+        self._styleChanged = True
 
     def styleGet(self, field):
         toSearch = [self, self._row, self._column, self._row._table]
@@ -203,23 +189,27 @@ class Cell(StyleUser):
         return (colWidth, rowHeight)
 
     def _getRect(self):
-        return pygame.Rect(self._getPt(), self._getSize())
+        if self.rect is None:
+            self.rect = pygame.Rect(self._getPt(), self._getSize())
+        return self.rect
 
     def _getUsableRect(self):
+        if self.usableRect is not None:
+            return self.usableRect
         pt = self._getPt()
         padding = self.styleGet('padding')
         usablePt = addPositions(pt, padding)
         size = self._getSize()
-        usableSize = tuple([size[i] - padding[i]*2 for i in 0,1])
-        return pygame.Rect(usablePt, usableSize)
+        usableSize = tuple([size[i] - padding[i] * 2 for i in (0, 1)])
+        self.usableRect = pygame.Rect(usablePt, usableSize)
+        return self.usableRect
 
     def _boxClicked(self, sender):
         self.setFocus(sender)
-        self._column.onClicked(self)
 
     def draw(self, surface):
-        if self._changed:
-            self._changed = False
+        if self._styleChanged:
+            self._styleChanged = False
             self._update()
         bgColour = self.styleGet('backColour')
         if bgColour is not None:
@@ -233,18 +223,22 @@ class Cell(StyleUser):
         finally:
             surface.set_clip(clipRect)
 
+
 class TextCell(Cell):
     def __init__(self, app, row, column):
         super(TextCell, self).__init__(app, row, column)
         textAlign = self.styleGet('textAlign')
-        self.textElement = TextElement(self.app, '', self.styleGet('font'), Location(CellAttachedPoint((0,0), self, textAlign), textAlign), colour=self.styleGet('foreColour'))
+        self.textElement = TextElement(
+            self.app, '', self.styleGet('font'),
+            Location(CellAttachedPoint((0, 0), self, textAlign), textAlign),
+            colour=self.styleGet('foreColour'))
         self.elements = [self.textElement]
         self._oldText = ''
 
     def setText(self, text):
         if text != self._oldText:
             self.textElement.setText(text)
-            self._changed = True
+            self._styleChanged = True
             self._oldText = text
 
     def _update(self):
@@ -256,13 +250,18 @@ class TextCell(Cell):
         self.textElement.setShadow(self.styleGet('hasShadow'))
         self.textElement.setShadowColour(self.styleGet('shadowColour'))
 
+
 class TextBoxCell(Cell):
     def __init__(self, app, row, column):
         super(TextBoxCell, self).__init__(app, row, column)
         textAlign = self.styleGet('textAlign')
-        self.inputBox = InputBox(self.app, Area(CellAttachedPoint((0,0), self,
-                textAlign), self._getUsableRect().size, textAlign),
-                font=self.styleGet('font'), colour=self.styleGet('foreColour'))
+        self.inputBox = InputBox(
+            self.app,
+            Area(
+                CellAttachedPoint((0, 0), self, textAlign),
+                self._getUsableRect().size, textAlign),
+            font=self.styleGet('font'),
+            colour=self.styleGet('foreColour'))
         self.inputBox.onEdit.addListener(self._valueChanged)
         self.elements = [self.inputBox]
         self._oldText = ''
@@ -283,15 +282,11 @@ class TextBoxCell(Cell):
             self.inputBox.onClick.removeListener(self._boxClicked)
         else:
             self.inputBox.onClick.addListener(self._boxClicked)
-            self.setFocus(None)
-
-    def lostFocus(self):
-        self.setFocus(None)
 
     def setText(self, text):
         if text != self._oldText:
             self.inputBox.setValue(text)
-            self._changed = True
+            self._styleChanged = True
             self._oldText = text
 
     def getText(self):
@@ -313,24 +308,25 @@ class TextBoxCell(Cell):
         self.inputBox.area.anchor = textAlign
         self.inputBox.area.point.attachedAt = textAlign
 
+
 class TextButtonCell(Cell):
     def __init__(self, app, row, column):
         super(TextButtonCell, self).__init__(app, row, column)
         textAlign = self.styleGet('textAlign')
-        pos = Location(CellAttachedPoint((0,0), self, textAlign), textAlign)
-        self.textButton = TextButton(self.app, pos, '', self.styleGet('font'), self.styleGet('foreColour'),
-                                     self.styleGet('hoverColour'))
+        pos = Location(CellAttachedPoint((0, 0), self, textAlign), textAlign)
+        self.textButton = TextButton(
+            self.app, pos, '', self.styleGet('font'),
+            self.styleGet('foreColour'), self.styleGet('hoverColour'))
         self.elements = [self.textButton]
         self._oldText = ''
 
     def setOnClick(self, onClick):
         self.textButton.onClick.addListener(onClick)
-        self._boxClicked(None)
 
     def setText(self, text):
         if text != self._oldText:
             self.textButton.setText(text)
-            self._changed = True
+            self._styleChanged = True
             self._oldText = text
 
     def _update(self):
@@ -345,34 +341,39 @@ class TextButtonCell(Cell):
 
 class CellAttachedPoint(AttachedPoint):
     def __init__(self, val, cell, attachedAt='topleft'):
-        super(CellAttachedPoint, self).__init__(val, cell._getUsableRect, attachedAt)
+        super(CellAttachedPoint, self).__init__(
+            val, cell._getUsableRect, attachedAt)
+
 
 class Value(object):
     def __init__(self, value):
         self._value = value
+
     def getVal(self):
         return self._value
 
+
 class Table(StyleUser):
-    def __init__(self, app, pos, rows=None, columns=None):
+    def __init__(self, app, pos, rows=None, columns=None, topPadding=0):
         defaultStyle = TableStyle()
-        defaultStyle.backColour = (255,255,255)
-        defaultStyle.padding = (5,5)
-        defaultStyle.foreColour = (0,0,0)
-        defaultStyle.hoverColour = (0,0,0)
+        defaultStyle.backColour = (255, 255, 255)
+        defaultStyle.padding = (5, 5)
+        defaultStyle.foreColour = (0, 0, 0)
+        defaultStyle.hoverColour = (0, 0, 0)
         defaultStyle.font = Font('FreeSans.ttf', 36)
         defaultStyle.textAlign = 'topleft'
         defaultStyle.hasShadow = False
         defaultStyle.shadowColour = colours.shadowDefault
         super(Table, self).__init__(app, defaultStyle)
         self.pos = pos
+        self.topPadding = topPadding
 
         self._columns = []
         self._rows = []
         self.elements = self._columns
 
         # Style Settings:
-        self._borderColour = (192,192,192)
+        self._borderColour = (192, 192, 192)
         self._borderWidth = 10
         self._defaultWidth = 200
         self.setDefaultHeight(Value(60))
@@ -385,9 +386,12 @@ class Table(StyleUser):
 
         self._sizes = {}
         self._appSize = None
+        app.screenManager.onResize.addListener(self.appResized)
 
-    def onClicked(self, sender):
-        self.setFocus(sender)
+    def appResized(self):
+        for row in self._rows:
+            for cell in row._cells:
+                cell.layoutChanged()
 
     def setDefaultHeight(self, height):
         self._defaultHeight = height
@@ -395,8 +399,8 @@ class Table(StyleUser):
     def styleUpdate(self):
         '''
         Informs the table that the style of some part of the table has changed.
-        This is provided because it is too expensive for every cell to check its
-        style every draw() call, so you must manually tell the table, row,
+        This is provided because it is too expensive for every cell to check
+        its style every draw() call, so you must manually tell the table, row,
         column or cell that the style has been updated before the update is
         reflected.
         '''
@@ -469,7 +473,8 @@ class Table(StyleUser):
         index = firstIndex
 
         for i in range(count):
-            self.addColumn(TextColumn(self.app, self, self._defaultWidth), index)
+            self.addColumn(
+                TextColumn(self.app, self, self._defaultWidth), index)
             if index is not None:
                 index += 1
 
@@ -493,20 +498,25 @@ class Table(StyleUser):
             self._columns[x]._anotherDeletedBefore()
         self._sizes = {}
 
-
     def rowCount(self):
         return len(self._rows)
 
     def columnCount(self):
         return len(self._columns)
 
+    def getRect(self, app):
+        assert app == self.app
+        return self._getRect()
+
     def _getRect(self):
-        r = pygame.Rect((0,0), self._getSize())
+        r = pygame.Rect((0, 0), self._getSize())
         self.pos.apply(self.app, r)
         return r
 
     def _getPt(self):
-        return self._getRect().topleft
+        x, y = self._getRect().topleft
+        y += self.topPadding
+        return x, y
 
     def _getSize(self, rowStart=0, rowEnd=None, colStart=0, colEnd=None):
         if rowEnd is None:
@@ -544,7 +554,7 @@ class Table(StyleUser):
 
     def _getColPt(self, index):
         pt = self._getPt()
-        offset = self._getSize(colStart=0,colEnd=index)
+        offset = self._getSize(colStart=0, colEnd=index)
         return (pt[0]+offset[0], pt[1])
 
     def draw(self, surface):
@@ -566,7 +576,8 @@ class Table(StyleUser):
             return
         # Draw Horizontals:
         # Use rect filling, because drawing lines usually round the ends
-        horRect = pygame.Rect((0,0), (self._getSize(rowStart=0, rowEnd=1)[0], width))
+        horRect = pygame.Rect(
+            (0, 0), (self._getSize(rowStart=0, rowEnd=1)[0], width))
         for row in self._rows:
             horRect.bottomleft = row._getPt()
             surface.fill(self._borderColour, horRect)
@@ -575,14 +586,14 @@ class Table(StyleUser):
 
         # Draw Verticals:
         # Use rect filling, because drawing lines usually round the ends
-        verRect = pygame.Rect((0,0), (width, self._getSize(colStart=0, colEnd = 1)[1]))
+        verRect = pygame.Rect(
+            (0, 0), (width, self._getSize(colStart=0, colEnd=1)[1]))
         for col in self._columns:
             verRect.topright = col._getPt()
             surface.fill(self._borderColour, verRect)
         verRect.left += col._getWidth() + width
         surface.fill(self._borderColour, verRect)
 
-
     def __getitem__(self, index):
         return self.getColumn(index)
 
@@ -591,4 +602,3 @@ class Table(StyleUser):
 
     def getColumn(self, index):
         return self._columns[index]
-
