@@ -19,7 +19,7 @@
 zone tallies onto the screen.'''
 
 import random
-
+import logging
 import pygame
 
 from trosnoth.gui.framework.elements import TextElement
@@ -28,10 +28,12 @@ import trosnoth.gui.framework.framework as framework
 from trosnoth.gui.framework.basics import Animation
 from trosnoth.utils.utils import timeNow
 
+log = logging.getLogger(__name__)
 
-class ZoneProgressBar(framework.CompoundElement):
+
+class GameProgressBar(framework.CompoundElement):
     def __init__(self, app, world, gameViewer):
-        super(ZoneProgressBar, self).__init__(app)
+        super(GameProgressBar, self).__init__(app)
         self.world = world
         self.gameViewer = gameViewer
         self.app = app
@@ -40,15 +42,15 @@ class ZoneProgressBar(framework.CompoundElement):
 
         colours = app.theme.colours
         self.black = colours.black
-        self.blue = colours.team1Mn_zone
-        self.red = colours.team2Mn_zone
+        self.blue = self.world.teams[0].shade(0.5, 1)
+        self.red = self.world.teams[1].shade(0.5, 1)
         self.grey = colours.zoneBarNeutral
 
         self.barFont = app.screenManager.fonts.zoneBarFont
 
         # Define a few constants to make things easier
         self.triangleLength = 25
-        self.sideDistance = 4
+        self.sideDistance = 0
         self.barHeight = self.gameViewer.zoneBarHeight
         self.textSpace = 130
         self.neutralTextSpace = 15
@@ -65,9 +67,17 @@ class ZoneProgressBar(framework.CompoundElement):
         self.xRight = None
         self.yTop = None
         self.yBottom = None
-        self.disruptAnimation = None
+        self.disruption_frame = None
+        self.disruption_frame_began = 0
         self.calculateSize()
 
+        self.blue_text = TextElement(
+            self.app, '', self.barFont, Location((0, 0), 'topright'), colour=colours.black)
+        self.neutral_text = TextElement(
+            self.app, '', self.barFont, Location((0, 0), 'midtop'), colour=colours.black)
+        self.red_text = TextElement(
+            self.app, '', self.barFont, Location((0, 0), 'topleft'), colour=colours.black)
+
     def calculateSize(self):
         minimapRect = self.gameViewer.miniMap.getRect()
         scaleFactor = self.app.screenManager.scaleFactor
@@ -91,12 +101,11 @@ class ZoneProgressBar(framework.CompoundElement):
         self.yTop = self.mapBottom - 1
         self.yBottom = self.yTop + self.barHeight
 
-        # Create the disrupted animation
-        self.disruptAnimation = self.createDisrupted()
-
     def draw(self, surface):
+        if not self.world.map:
+            return
         self.calculateSize()
-        super(ZoneProgressBar, self).draw(surface)
+        super(GameProgressBar, self).draw(surface)
 
         if self.disrupt and random.random() < 0.2:
             disrupt = True
@@ -135,26 +144,23 @@ class ZoneProgressBar(framework.CompoundElement):
         ]
 
         # Get the information we need
-        blueZones = self.world.teams[0].numOrbsOwned
-        redZones = self.world.teams[1].numOrbsOwned
-        totalZones = len(self.world.zones)
-        neutralZones = totalZones - blueZones - redZones
+        blueScore = self.getBlueScore()
+        blueProportion = self.getBlueProportion()
+        redScore = self.getRedScore()
+        redProportion = self.getRedProportion()
+        neutralScore = self.getNeutralScore()
+        neutralProportion = self.getNeutralProportion()
 
-        if blueZones == 0 or redZones == 0:
-            gameOver = True
-        else:
-            gameOver = False
+        gameOver = self.isGameOver()
 
         # Define the coordinates for the dynamic shapes
         mutableBarLength = xRight - xLeft
-        if redZones > 0:
-            blueBarLength = round(mutableBarLength * ((blueZones * 1.0) /
-                    totalZones))
+        if redProportion > 0:
+            blueBarLength = int(round(mutableBarLength * blueProportion))
         else:
             blueBarLength = mutableBarLength
-        if blueZones > 0:
-            redBarLength = round(mutableBarLength * ((redZones * 1.0) /
-                    totalZones))
+        if blueProportion > 0:
+            redBarLength = int(round(mutableBarLength * redProportion))
         else:
             redBarLength = mutableBarLength
 
@@ -183,35 +189,42 @@ class ZoneProgressBar(framework.CompoundElement):
         ]
 
         # Draw the two triangles on the sides
-        if blueZones > 0:
+        if blueProportion > 0:
             pygame.draw.polygon(surface, self.blue, blueTriangle, 0)
-        else:
+        elif redProportion > 0:
             pygame.draw.polygon(surface, self.red, blueTriangle, 0)
-        if redZones > 0:
-            pygame.draw.polygon(surface, self.red, redTriangle, 0)
         else:
+            pygame.draw.polygon(surface, self.grey, blueTriangle, 0)
+
+        if redProportion > 0:
+            pygame.draw.polygon(surface, self.red, redTriangle, 0)
+        elif blueProportion > 0:
             pygame.draw.polygon(surface, self.blue, redTriangle, 0)
+        else:
+            pygame.draw.polygon(surface, self.grey, redTriangle, 0)
 
         # Draw the team colours
-        if blueZones > 0:
+        if blueProportion > 0:
             pygame.draw.polygon(surface, self.blue, blueBar, 0)
-        if redZones > 0:
+        if redProportion > 0:
             pygame.draw.polygon(surface, self.red, redBar, 0)
-        if neutralZones > 0 and not gameOver:
+        if neutralProportion > 0 and (blueScore > 0) == (redScore > 0):
+            allNeutral = True
             pygame.draw.polygon(surface, self.grey, greyBar, 0)
+        else:
+            allNeutral = False
 
         # Draw the black seperator line(s)
         if not gameOver:
             pygame.draw.line(surface, self.black, (xFirstBar, yTop),
                     (xFirstBar, yBottom), self.width)
-            if neutralZones != 0:
+            if neutralScore != 0:
                 pygame.draw.line(surface, self.black, (xSecondBar, yTop),
                         (xSecondBar, yBottom), self.width)
 
         # Draw the disruption
         if disrupt:
-            surface.blit(self.disruptAnimation.getImage(), (self.xFarLeft,
-                    self.yTop))
+            surface.blit(self.get_disruption_frame(), (self.xFarLeft, self.yTop))
 
         # Draw the border last so that it goes on top
         colours = self.app.theme.colours
@@ -219,62 +232,140 @@ class ZoneProgressBar(framework.CompoundElement):
         pygame.draw.line(surface, colours.minimapBorder, (self.mapLeft,
                 self.mapBottom - 1), (self.mapRight, self.mapBottom - 1), 2)
 
-        # Define the necessary text
-        self.blueText = TextElement(self.app, '', self.barFont,
-                Location((xFirstBar - 5, yText), 'topright'),
-                colour=colours.black)
-        self.neutralText = TextElement(self.app, '', self.barFont,
-                Location((xNeutral, yText), 'midtop'), colour=colours.black)
-        self.redText = TextElement(self.app, '', self.barFont,
-                Location((xSecondBar + 7, yText), 'topleft'),
-                colour=colours.black)
+        self.blue_text.setPos(Location((xFirstBar - 5, yText), 'topright'))
+        self.neutral_text.setPos(Location((xNeutral, yText), 'midtop'))
+        self.red_text.setPos(Location((xSecondBar + 7, yText), 'topleft'))
 
         if not disrupt:
-            blueString = str(blueZones)
-            redString = str(redZones)
+            blueString = self.getScoreText(blueScore)
+            redString = self.getScoreText(redScore)
 
-            if xSecondBar - xFirstBar < self.neutralTextSpace:
-                neutralString = ""
+            if xSecondBar - xFirstBar > self.neutralTextSpace or allNeutral:
+                neutralString = self.getScoreText(neutralScore)
             else:
-                neutralString = str(neutralZones)
+                neutralString = ''
 
-            self.blueText.setText(blueString)
-            self.redText.setText(redString)
-            self.neutralText.setText(neutralString)
+            self.blue_text.setText(blueString)
+            self.red_text.setText(redString)
+            self.neutral_text.setText(neutralString)
         else:
-            self.blueText.setText('')
-            self.redText.setText('')
-            self.neutralText.setText('')
+            self.blue_text.setText('')
+            self.red_text.setText('')
+            self.neutral_text.setText('')
 
         # Draw the text
-        if blueZones > 0:
-            self.blueText.draw(surface)
-        if redZones > 0:
-            self.redText.draw(surface)
-        if neutralZones > 0 and not gameOver:
-            self.neutralText.draw(surface)
+        self.blue_text.draw(surface)
+        self.red_text.draw(surface)
+        self.neutral_text.draw(surface)
 
-    def createDisrupted(self):
-        screens = []
-        for a in range(0,4):
-            screen = (pygame.surface.Surface((self.xFarRight - self.xFarLeft,
-                                              self.gameViewer.zoneBarHeight)))
-            x = y = 0
-            xL = 1
-            xR = self.xFarRight - self.xFarLeft - 1
-            rect = pygame.rect.Rect(0,0,2,2)
-            disruptColours = [self.red, self.blue]
-            while y < self.gameViewer.zoneBarHeight:
-                while x < xR:
-                    rect.left = x
-                    rect.top = y
-                    pygame.draw.rect(screen, random.choice(disruptColours),
-                            rect, 0)
-                    x += 2
-                xL += 2
-                xR -= 2
-                x = xL
-                y += 2
-            screen.set_colorkey((0,0,0))
-            screens.append(screen)
-        return Animation(0.1, timeNow, *screens)
+    def get_disruption_frame(self):
+        now = timeNow()
+        if not self.disruption_frame:
+            self.disruption_frame = pygame.Surface((
+                self.xFarRight - self.xFarLeft, self.gameViewer.zoneBarHeight))
+        elif now <= self.disruption_frame_began + 0.1:
+            return self.disruption_frame
+
+        self.disruption_frame_began = now
+        static = self.app.theme.sprites.static
+        area = static.get_rect()
+        area.left = random.randrange(static.get_width())
+        area.top = random.randrange(static.get_height())
+        x = y = 0
+        while y < self.disruption_frame.get_height():
+            while x < self.disruption_frame.get_width():
+                r = self.disruption_frame.blit(static, (x, y), area)
+                x += r.width
+                area.left = 0
+            x = 0
+            y += r.height
+            area.top = 0
+        return self.disruption_frame
+
+    def getScoreText(self, score):
+        raise NotImplementedError
+
+    def getBlueScore(self):
+        raise NotImplementedError
+
+    def getRedScore(self):
+        raise NotImplementedError
+
+    def getNeutralScore(self):
+        raise NotImplementedError
+
+    def getBlueProportion(self):
+        raise NotImplementedError
+
+    def getNeutralProportion(self):
+        raise NotImplementedError
+
+    def getRedProportion(self):
+        raise NotImplementedError
+
+    def isGameOver(self):
+        raise NotImplementedError
+
+
+class ZoneProgressBar(GameProgressBar):
+    def getScoreText(self, score):
+        if score == 0:
+            return ''
+        return str(score)
+
+    def getBlueScore(self):
+        return self.world.teams[0].numZonesOwned
+
+    def getRedScore(self):
+        return self.world.teams[1].numZonesOwned
+
+    def getNeutralScore(self):
+        return len(self.world.zones) - self.getBlueScore() - self.getRedScore()
+
+    def getBlueProportion(self):
+        if not self.world.zones:
+            return 0.0
+        return self.getBlueScore() / len(self.world.zones)
+
+    def getNeutralProportion(self):
+        if not self.world.zones:
+            return 0.0
+        return self.getNeutralScore() / len(self.world.zones)
+
+    def getRedProportion(self):
+        if not self.world.zones:
+            return 0.0
+        return self.getRedScore() / len(self.world.zones)
+
+    def isGameOver(self):
+        return self.getBlueScore() == 0 or self.getRedScore() == 0
+
+
+class FrontLineProgressBar(GameProgressBar):
+    def getScoreText(self, score):
+        return str(score)
+
+    def getBlueScore(self):
+        return str(self.world.scoreboard.teamScores[self.world.teams[0]])
+
+    def getRedScore(self):
+        return str(self.world.scoreboard.teamScores[self.world.teams[1]])
+
+    def getNeutralScore(self):
+        return 0
+
+    def getBlueProportion(self):
+        frontLine = self.world.uiOptions.getFrontLine()
+        if frontLine is None:
+            return 0.5
+        mapWidth = self.world.layout.worldSize[0]
+        return max(0.0, min(1.0, frontLine / (mapWidth + 0.0)))
+
+    def getNeutralProportion(self):
+        return 0.0
+
+    def getRedProportion(self):
+        return 1 - self.getBlueProportion()
+
+    def isGameOver(self):
+        return False
