@@ -1,5 +1,5 @@
 # Trosnoth (UberTweak Platform Game)
-# Copyright (C) 2006-2011 Joshua D Bartlett
+# Copyright (C) 2006-2012 Joshua D Bartlett
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,10 +15,14 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301, USA.
 
+import logging
 import struct
 
 from trosnoth.utils.message import MessageBase as Message
 
+log = logging.getLogger(__name__)
+
+
 ##################################
 # Message-related errors
 ##################################
@@ -28,26 +32,29 @@ class NetworkMessageError(Exception):
     An attempt was made to process a badly formatted network message.
     '''
 
+
 class MessageTypeError(NetworkMessageError):
     '''
     An attempt was made to build a NetworkMessage object from a string which
     does not represent a valid network message type.
     '''
 
+
 class MessageContentsError(NetworkMessageError):
     '''
     An attempt was made to build a NetworkMessage object from a badly formatted
     message string.
     '''
 
+
 ##################################
 # Network message metaclass
 ##################################
 
 class NetworkMessageClass(type):
     '''
-    Metaclass which automatically registers subclasses of NetworkMessage
-    provided that they define a .idString class attribute.
+    Metaclass which performs declaration-time checking of attributes of
+    NetworkMessage subclasses.
     '''
     def __new__(cls, name, bases, dictn):
         # Test for a single string cls.fields and make tuple.
@@ -62,13 +69,14 @@ class NetworkMessageClass(type):
         # Do the real building.
         result = type.__new__(cls, name, bases, dictn)
 
-        if hasattr(result, 'idString'):
+        if getattr(result, 'idString'):
             # Validate the class attributes.
-            if not isinstance(result.idString, str):
-                raise TypeError('Message type idString must be string')
+            if not isinstance(result.idString, bytes):
+                raise TypeError('Message type idString must be byte string')
             if len(result.idString) != 4:
-                raise ValueError('id string %r does not have 4 characters' %
-                        (result.idString,))
+                raise ValueError(
+                    'id string %r does not have 4 characters' % (
+                        result.idString,))
 
             # Check for the other 2 required attributes.
             if not hasattr(result, 'packspec'):
@@ -78,6 +86,7 @@ class NetworkMessageClass(type):
 
         return result
 
+
 #############################
 # Network message registries
 #############################
@@ -89,20 +98,20 @@ class MessageCollection(object):
             self._registerType(msgType)
 
     def __iter__(self):
-        return self._msgTypes.itervalues()
+        return iter(self._msgTypes.values())
 
     def _registerType(self, msgType):
         '''
         Registers the given network message type.
         '''
-        if not isinstance(msgType.idString, str):
+        if not isinstance(msgType.idString, bytes):
             raise TypeError('Message type idString must be string')
         if len(msgType.idString) != 4:
-            raise ValueError('id string %r does not have 4 characters' %
-                    (msgType.idString,))
+            raise ValueError('id string %r does not have 4 characters' % (
+                msgType.idString,))
         if msgType.idString in self._msgTypes:
-            raise KeyError('id string %r already used for %s' %
-                    (msgType.idString, self._msgTypes[msgType.idString]))
+            raise KeyError('id string %r already used for %s' % (
+                msgType.idString, self._msgTypes[msgType.idString]))
 
         self._msgTypes[msgType.idString] = msgType
 
@@ -110,7 +119,7 @@ class MessageCollection(object):
         '''
         Builds a NetworkMessage object from the packed string.
         '''
-        if not isinstance(input, str):
+        if not isinstance(input, bytes):
             raise TypeError('%s is not a string' % (input,))
 
         # Look up the id string to find the message type.
@@ -118,15 +127,16 @@ class MessageCollection(object):
         try:
             msgType = self._msgTypes[idString]
         except KeyError:
-            raise MessageTypeError('bad id string: %r' % (idString))
+            raise MessageTypeError('bad id string: %r' % (idString,))
 
         return msgType._build(input[4:])
 
+
 ############################
 # Network message class
 ############################
 
-class NetworkMessage(Message):
+class NetworkMessage(Message, metaclass=NetworkMessageClass):
     '''
     Allows network messages to be defined by:
         class CustomNetMsg(NetworkMessage):
@@ -140,20 +150,27 @@ class NetworkMessage(Message):
             character may optionally be * meaning that the rest of the message
             is considered to be a single string.
     '''
-    __metaclass__ = NetworkMessageClass
+    idString = None
+    trace = False       # Set to True to see where a message goes
+    pumpAllEvents = False
 
     def __init__(self, *args, **kwargs):
         # Check that all kwargs are valid keys.
-        for k in kwargs.iterkeys():
+        for k in kwargs.keys():
             if k not in self.fields:
-                raise TypeError('%s() got unexpected keyword argument %r' %
-                        (self.__class__, k))
+                raise TypeError('%s() got unexpected keyword argument %r' % (
+                    self.__class__, k))
 
         # Initialise attributes based on arguments.
         Message.__init__(self, *args, **kwargs)
 
         # Check that packing can be performed.
-        self.pack()
+        if self.idString:
+            self.pack()
+
+    def tracePoint(self, obj, method):
+        if self.trace:
+            log.warning('%s: %s (%s)', self, method, obj)
 
     def pack(self):
         '''
@@ -169,19 +186,20 @@ class NetworkMessage(Message):
 
         # Special processing if there's a * in self.packspec.
         if self.packspec.endswith('*'):
-            if not isinstance(values[-1], str):
-                raise TypeError('%s field must be string' % (self.fields[-1]))
+            if not isinstance(values[-1], bytes):
+                raise TypeError('%s field must be byte string' % (self.fields[-1]))
             suffix = values.pop()
             packspec = self.packspec[:-1]
         else:
-            suffix = ''
+            suffix = b''
             packspec = self.packspec
 
         # Perform the packing.
         try:
             result = struct.pack('!' + packspec, *values)
         except:
-            raise Exception, '%s message couldn\'t pack string %s' % (self.idString, str(values))
+            raise Exception('%s message couldn\'t pack string %s' % (
+                self.idString, str(values)))
 
         return self.idString + result + suffix
 
@@ -192,8 +210,8 @@ class NetworkMessage(Message):
         network message string.
         '''
         packspec = '!' + cls.packspec
-        hasStar = packspec.endswith('*')
-        if hasStar:
+        hasCoin = packspec.endswith('*')
+        if hasCoin:
             packspec = packspec[:-1]
             size = struct.calcsize(packspec)
             suffix = source[size:]
@@ -202,17 +220,17 @@ class NetworkMessage(Message):
             # Verify size.
             size = struct.calcsize(packspec)
             if len(source) != size:
-                raise MessageContentsError('bad message length for %s' %
-                        cls.__name__)
+                raise MessageContentsError(
+                    'bad message length for %s' % cls.__name__)
 
         # Do the actual unpacking.
         try:
             values = struct.unpack(packspec, source)
-        except struct.error, E:
-            raise MessageContentsError('bad %s string: %s' %
-                    (cls.__name__, E.args[0]))
+        except struct.error as E:
+            raise MessageContentsError(
+                'bad %s string: %s' % (cls.__name__, E.args[0]))
 
         # Build the message.
-        if hasStar:
+        if hasCoin:
             values += (suffix,)
         return cls(*values)
